@@ -42,8 +42,8 @@
 #include <math.h>
 #include <stdio.h>
 
-static void     eel_background_class_init          (gpointer       klass);
-static void     eel_background_init                (gpointer       object,
+static void     eel_background_class_init                (gpointer       klass);
+static void     eel_background_init                      (gpointer       object,
 							  gpointer       klass);
 static void     eel_background_finalize                  (GObject       *object);
 static void     eel_background_start_loading_image       (EelBackground *background,
@@ -61,7 +61,10 @@ enum {
 	LAST_SIGNAL
 };
 
-#define	RESET_BACKGROUND_MAGIC_IMAGE_NAME "reset.png"
+/* This is the size of the GdkRGB dither matrix, in order to avoid
+ * bad dithering when tiling the gradient
+ */
+#define GRADIENT_PIXMAP_TILE_SIZE 128
 
 static guint signals[LAST_SIGNAL];
 
@@ -74,6 +77,8 @@ struct EelBackgroundDetails {
 
 	gboolean is_solid_color;
 	GdkColor solid_color;
+
+	gboolean constant_size;
 	
 	char *image_uri;
 	GdkPixbuf *image;
@@ -92,6 +97,13 @@ struct EelBackgroundDetails {
 	int image_rect_y;
 	int image_rect_width;
 	int image_rect_height;
+
+	/* Realized data: */
+	GdkPixmap *background_pixmap;
+	int background_entire_width;
+	int background_entire_height;
+	GdkColor background_color;
+	gboolean background_changes_with_size;
 };
 
 static void
@@ -104,42 +116,46 @@ eel_background_class_init (gpointer klass)
 	background_class = EEL_BACKGROUND_CLASS (klass);
 
 	signals[APPEARANCE_CHANGED] =
-		gtk_signal_new ("appearance_changed",
-				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelBackgroundClass,
-						   appearance_changed),
-				gtk_marshal_VOID__VOID,
-				G_TYPE_NONE,
-				0);
+		g_signal_new ("appearance_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       appearance_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	signals[SETTINGS_CHANGED] =
-		gtk_signal_new ("settings_changed",
-				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelBackgroundClass,
-						   settings_changed),
-				gtk_marshal_VOID__VOID,
-				G_TYPE_NONE,
-				0);
+		g_signal_new ("settings_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       settings_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	signals[IMAGE_LOADING_DONE] =
-		gtk_signal_new ("image_loading_done",
-				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelBackgroundClass,
-						   image_loading_done),
-				gtk_marshal_VOID__BOOLEAN,
-				G_TYPE_NONE,
-				1,
-				G_TYPE_BOOLEAN);
+		g_signal_new ("image_loading_done",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       image_loading_done),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOOLEAN,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_BOOLEAN);
 	signals[RESET] =
-		gtk_signal_new ("reset",
-				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelBackgroundClass,
-						   reset),
-				gtk_marshal_VOID__VOID,
-				G_TYPE_NONE,
-				0);
+		g_signal_new ("reset",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       reset),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 
 	object_class->finalize = eel_background_finalize;
 }
@@ -149,9 +165,10 @@ eel_background_init (gpointer object, gpointer klass)
 {
 	EelBackground *background;
 
-	background = EEL_BACKGROUND(object);
+	background = EEL_BACKGROUND (object);
 
 	background->details = g_new0 (EelBackgroundDetails, 1);
+	background->details->constant_size = FALSE;
 }
 
 /* The safe way to clear an image from a background is:
@@ -184,9 +201,23 @@ eel_background_finalize (GObject *object)
 	eel_background_remove_current_image (background);
 	g_free (background->details);
 
+	if (background->details->background_pixmap != NULL) {
+		g_object_unref (background->details->background_pixmap);
+		background->details->background_pixmap = NULL;
+	}
+	
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
+void
+eel_background_set_is_constant_size (EelBackground *background,
+				     gboolean       constant_size)
+{
+	g_return_if_fail (EEL_IS_BACKGROUND (background));
+	
+	background->details->constant_size = constant_size;
+}
+
 EelBackgroundImagePlacement
 eel_background_get_image_placement (EelBackground *background)
 {
@@ -197,7 +228,7 @@ eel_background_get_image_placement (EelBackground *background)
 
 static gboolean
 eel_background_set_image_placement_no_emit (EelBackground *background,
-						 EelBackgroundImagePlacement new_placement)
+					    EelBackgroundImagePlacement new_placement)
 {
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
 
@@ -219,14 +250,14 @@ eel_background_set_image_placement_no_emit (EelBackground *background,
 }
 
 void
-eel_background_set_image_placement (EelBackground *background,
-					 EelBackgroundImagePlacement new_placement)
+eel_background_set_image_placement (EelBackground              *background,
+				    EelBackgroundImagePlacement new_placement)
 {
 	if (eel_background_set_image_placement_no_emit (background, new_placement)) {
-		gtk_signal_emit (GTK_OBJECT (background),
-				 signals[SETTINGS_CHANGED]);
-		gtk_signal_emit (GTK_OBJECT (background),
-				 signals[APPEARANCE_CHANGED]);
+		g_signal_emit (G_OBJECT (background),
+			       signals[SETTINGS_CHANGED], 0);
+		g_signal_emit (G_OBJECT (background),
+			       signals[APPEARANCE_CHANGED], 0);
 	}
 }
 
@@ -490,9 +521,9 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 			break;
 		case EEL_BACKGROUND_SCALED_ASPECT:
 			eel_gdk_scale_to_fit_factor (background->details->image_width_unscaled,
-							  background->details->image_height_unscaled,
-							  dest_width, dest_height,
-							  &fit_width, &fit_height);
+						     background->details->image_height_unscaled,
+						     dest_width, dest_height,
+						     &fit_width, &fit_height);
 			if (image_width != fit_width || image_height != fit_height) {
 				if (cur_scaled) {
 					reload_image = TRUE;
@@ -529,6 +560,138 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 	}
 }
 
+static gboolean
+get_pixmap_size (EelBackground    *background,
+		 int               entire_width,
+		 int               entire_height,
+		 int              *pixmap_width,
+		 int              *pixmap_height,
+		 gboolean         *changes_with_size)
+{
+	*pixmap_width = 0;	
+	*pixmap_height = 0;
+	*changes_with_size = ! background->details->constant_size;
+	
+	if (background->details->image == NULL) {
+		if (background->details->is_solid_color) {
+			*changes_with_size = FALSE;
+			return FALSE;
+		}
+		if (background->details->gradient_is_horizontal) {
+			*pixmap_width = entire_width;
+			*pixmap_height = GRADIENT_PIXMAP_TILE_SIZE;
+		} else {
+			*pixmap_width = GRADIENT_PIXMAP_TILE_SIZE;
+			*pixmap_height = entire_height;
+		}
+		return TRUE;
+	} else if (!eel_background_is_image_load_in_progress (background)) {
+		switch (background->details->image_placement) {
+		case EEL_BACKGROUND_TILED:
+			if (background->details->is_solid_color || !gdk_pixbuf_get_has_alpha (background->details->image)) {
+				*pixmap_width = background->details->image_width_unscaled;
+				*pixmap_height = background->details->image_height_unscaled;
+				*changes_with_size = FALSE;
+			} else {
+				if (background->details->gradient_is_horizontal) {
+					*pixmap_width = entire_width;
+					*pixmap_height = background->details->image_height_unscaled;
+				} else {
+					*pixmap_width = background->details->image_width_unscaled;
+					*pixmap_height = entire_height;
+				}
+			}
+			break;
+		case EEL_BACKGROUND_CENTERED:
+		case EEL_BACKGROUND_SCALED:
+		case EEL_BACKGROUND_SCALED_ASPECT:
+			*pixmap_width = entire_width;
+			*pixmap_height = entire_height;
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
+static void
+eel_background_unrealize (EelBackground *background)
+{
+	if (background->details->background_pixmap != NULL) {
+		g_object_unref (background->details->background_pixmap);
+		background->details->background_pixmap = NULL;
+	}
+	background->details->background_entire_width = 0;
+	background->details->background_entire_height = 0;
+}
+
+static void
+eel_background_ensure_realized (EelBackground *background, GdkWindow *window,
+				int entire_width, int entire_height)
+{
+	int pixmap_width, pixmap_height;
+	char *start_color_spec;
+ 	GdkPixmap *pixmap;
+	GdkGC *gc;
+
+	start_color_spec = eel_gradient_get_start_color_spec (background->details->color);
+	eel_gdk_color_parse_with_white_default (start_color_spec, &background->details->background_color);
+	g_free (start_color_spec);
+
+	/* If the pixmap doesn't change with the window size, never update
+	 * it again.
+	 */
+	if (background->details->background_pixmap != NULL &&
+	    !background->details->background_changes_with_size) {
+		return;
+	}
+
+	/* If the window size is the same as last time, don't update */
+	if (entire_width == background->details->background_entire_width &&
+	    entire_height == background->details->background_entire_height) {
+		return;
+	}
+
+	if (background->details->background_pixmap != NULL) {
+		g_object_unref (background->details->background_pixmap);
+		background->details->background_pixmap = NULL;
+	}
+	
+	if (get_pixmap_size (background, entire_width, entire_height,
+			      &pixmap_width, &pixmap_height, &background->details->background_changes_with_size)) {
+		pixmap = gdk_pixmap_new (window, pixmap_width, pixmap_height, -1);
+		gc = gdk_gc_new (pixmap);
+		eel_background_pre_draw (background,  entire_width, entire_height);
+		eel_background_draw (background, pixmap, gc,
+				     0, 0, 0, 0,
+				     pixmap_width, pixmap_height);
+		g_object_unref (gc);
+		background->details->background_pixmap = pixmap;
+	}
+
+	background->details->background_entire_width = entire_width;
+	background->details->background_entire_height = entire_height;
+}
+
+GdkPixmap *
+eel_background_get_pixmap_and_color (EelBackground *background,
+				     GdkWindow     *window,
+				     int            entire_width,
+				     int            entire_height,
+				     GdkColor      *color,
+				     gboolean      *changes_with_size)
+{
+	eel_background_ensure_realized (background, window, entire_width, entire_height);
+	
+	*color = background->details->background_color;
+	*changes_with_size = background->details->background_changes_with_size;
+	
+	if (background->details->background_pixmap != NULL) {
+		return g_object_ref (background->details->background_pixmap);
+	} 
+	return NULL;
+}
+
 void
 eel_background_pre_draw (EelBackground *background, int entire_width, int entire_height)
 {
@@ -588,10 +751,10 @@ eel_background_draw (EelBackground *background,
 
 void
 eel_background_draw_to_drawable (EelBackground *background,
-				      GdkDrawable *drawable, GdkGC *gc,
-				      int drawable_x, int drawable_y,
-				      int drawable_width, int drawable_height,
-				      int entire_width, int entire_height)
+				 GdkDrawable *drawable, GdkGC *gc,
+				 int drawable_x, int drawable_y,
+				 int drawable_width, int drawable_height,
+				 int entire_width, int entire_height)
 {
 	eel_background_pre_draw (background, entire_width, entire_height);
 	eel_background_draw (background, drawable, gc,
@@ -602,13 +765,13 @@ eel_background_draw_to_drawable (EelBackground *background,
 
 void
 eel_background_draw_to_pixbuf (EelBackground *background,
-				    GdkPixbuf *pixbuf,
-				    int pixbuf_x,
-				    int pixbuf_y,
-				    int pixbuf_width,
-				    int pixbuf_height,
-				    int entire_width,
-				    int entire_height)
+			       GdkPixbuf *pixbuf,
+			       int pixbuf_x,
+			       int pixbuf_y,
+			       int pixbuf_width,
+			       int pixbuf_height,
+			       int entire_width,
+			       int entire_height)
 {
 	GnomeCanvasBuf fake_buffer;
 
@@ -618,9 +781,9 @@ eel_background_draw_to_pixbuf (EelBackground *background,
 	canvas_buf_from_pixbuf (&fake_buffer, pixbuf, pixbuf_x, pixbuf_y, pixbuf_width, pixbuf_height);
 
 	eel_background_draw_to_canvas (background,
-					    &fake_buffer,
-					    entire_width,
-					    entire_height);
+				       &fake_buffer,
+				       entire_width,
+				       entire_height);
 }
 
 /* fill the canvas buffer with a tiled pixbuf */
@@ -701,9 +864,9 @@ eel_background_draw_aa (EelBackground *background, GnomeCanvasBuf *buffer)
 
 void
 eel_background_draw_to_canvas (EelBackground *background,
-				    GnomeCanvasBuf *buffer,
-				    int entire_width,
-				    int entire_height)
+			       GnomeCanvasBuf *buffer,
+			       int entire_width,
+			       int entire_height)
 {
 	eel_background_pre_draw (background, entire_width, entire_height);
 	eel_background_draw_aa (background, buffer);
@@ -727,7 +890,7 @@ eel_background_get_image_uri (EelBackground *background)
 
 static gboolean
 eel_background_set_color_no_emit (EelBackground *background,
-			       const char *color)
+				  const char *color)
 {
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
 
@@ -743,20 +906,20 @@ eel_background_set_color_no_emit (EelBackground *background,
 
 void
 eel_background_set_color (EelBackground *background,
-			       const char *color)
+			  const char *color)
 {
 	if (eel_background_set_color_no_emit (background, color)) {
-		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
+		g_signal_emit (G_OBJECT (background), signals[SETTINGS_CHANGED], 0);
 		if (!eel_background_image_totally_obscures (background)) {
-			gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
+			g_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED], 0);
 		}
 	}
 }
 
 static void
 eel_background_load_image_callback (GnomeVFSResult error,
-					 GdkPixbuf *pixbuf,
-					 gpointer callback_data)
+				    GdkPixbuf *pixbuf,
+				    gpointer callback_data)
 {
 	EelBackground *background;
 
@@ -774,11 +937,11 @@ eel_background_load_image_callback (GnomeVFSResult error,
 		background->details->image_height_unscaled = gdk_pixbuf_get_height (pixbuf);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE],
-			 pixbuf != NULL || background->details->image_uri == NULL);
+	g_signal_emit (G_OBJECT (background), signals[IMAGE_LOADING_DONE], 0,
+		       pixbuf != NULL || background->details->image_uri == NULL);
 
 	if (background->details->emit_after_load) {
-		gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
+		g_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED], 0);
 	}
 }
 
@@ -794,7 +957,7 @@ eel_background_cancel_loading_image (EelBackground *background)
 	if (eel_background_is_image_load_in_progress (background)) {
 		eel_cancel_gdk_pixbuf_load (background->details->load_image_handle);
 		background->details->load_image_handle = NULL;
-		gtk_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE], FALSE);
+		g_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE], 0, FALSE);
 	}
 }
 
@@ -821,9 +984,9 @@ eel_background_start_loading_image (EelBackground *background, gboolean emit_app
 
 static gboolean
 eel_background_set_image_uri_helper (EelBackground *background,
-					const char *image_uri,
-					gboolean emit_setting_change,
-					gboolean emit_appearance_change,
+				     const char *image_uri,
+				     gboolean emit_setting_change,
+				     gboolean emit_appearance_change,
 				     gboolean load_async)
 {
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
@@ -845,7 +1008,7 @@ eel_background_set_image_uri_helper (EelBackground *background,
 	eel_background_start_loading_image (background, emit_appearance_change, load_async);
 	
 	if (emit_setting_change) {
-		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
+		g_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED], 0);
 	}
 
 	return TRUE;
@@ -866,7 +1029,10 @@ eel_background_set_image_uri_sync (EelBackground *background, const char *image_
 static void
 set_image_and_color_image_loading_done_callback (EelBackground *background, gboolean successful_load, char *color)
 {
-	gtk_signal_disconnect_by_func (GTK_OBJECT (background), G_CALLBACK (set_image_and_color_image_loading_done_callback), (gpointer) color);
+	g_signal_handlers_disconnect_by_func (
+		G_OBJECT (background),
+		G_CALLBACK (set_image_and_color_image_loading_done_callback),
+		(gpointer) color);
 
 	eel_background_set_color_no_emit (background, color);
 
@@ -875,8 +1041,8 @@ set_image_and_color_image_loading_done_callback (EelBackground *background, gboo
 	/* We always emit , even if the color didn't change, because the image change
 	 * relies on us doing it here.
 	 */
-	gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
-	gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
+	g_signal_emit (G_OBJECT (background), signals[SETTINGS_CHANGED], 0);
+	g_signal_emit (G_OBJECT (background), signals[APPEARANCE_CHANGED], 0);
 }
 
 /* Use this fn to set both the image and color and avoid flash. The color isn't
@@ -919,20 +1085,15 @@ eel_background_set_image_uri_and_color (EelBackground *background, const char *i
 
 void
 eel_background_receive_dropped_background_image (EelBackground *background,
-						      const char *image_uri)
+						 const char *image_uri)
 {
-	/* Special case the reset-background image by file name. */
-	if (eel_str_has_suffix (image_uri, "/" RESET_BACKGROUND_MAGIC_IMAGE_NAME)) {
-		eel_background_reset (background);
-	} else {
-		/* Currently, we only support tiled images. So we set the placement.
-		 * We rely on eel_background_set_image_uri_and_color to emit
-		 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
-		 */
-		eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
-
-		eel_background_set_image_uri_and_color (background, image_uri, NULL);
-	}
+	/* Currently, we only support tiled images. So we set the placement.
+	 * We rely on eel_background_set_image_uri_and_color to emit
+	 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
+	 */
+	eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
+	
+	eel_background_set_image_uri_and_color (background, image_uri, NULL);
 }
 
 static void
@@ -987,7 +1148,7 @@ eel_background_reset (EelBackground *background)
 {
 	g_return_if_fail (EEL_IS_BACKGROUND (background));
 
-	gtk_signal_emit (GTK_OBJECT (background), signals[RESET]);
+	g_signal_emit (GTK_OBJECT (background), signals[RESET], 0);
 }
 
 static void
@@ -995,6 +1156,8 @@ draw_background_callback (GnomeCanvas *canvas, GdkDrawable *drawable,
 			  int x, int y, int width, int height)
 {
 	EelBackground *background;
+	GdkGCValuesMask value_mask;
+	GdkGCValues gc_values;
 	GdkGC *gc;
 
 	background = eel_get_widget_background (GTK_WIDGET (canvas));
@@ -1002,18 +1165,34 @@ draw_background_callback (GnomeCanvas *canvas, GdkDrawable *drawable,
 		return;
 	}
 
-	eel_background_pre_draw (background,
-				 GTK_WIDGET (canvas)->allocation.width,
-				 GTK_WIDGET (canvas)->allocation.height);
+	
+	eel_background_ensure_realized (background, GTK_WIDGET (canvas)->window,
+					MAX (GTK_LAYOUT (canvas)->hadjustment->upper, GTK_WIDGET (canvas)->allocation.width),
+					MAX (GTK_LAYOUT (canvas)->vadjustment->upper, GTK_WIDGET (canvas)->allocation.height));
+
 
 	/* Create a new gc each time.
 	 * If this is a speed problem, we can create one and keep it around,
 	 * but it's a bit more complicated to ensure that it's always compatible
 	 * with whatever drawable is passed in.
 	 */
-	gc = gdk_gc_new (drawable);
-	eel_background_draw (background, drawable, gc, x, y, 0, 0, width, height);
-	gdk_gc_unref (gc);
+	if (background->details->background_pixmap != NULL) {
+		gc_values.tile = background->details->background_pixmap;
+		gc_values.ts_x_origin = -x;
+		gc_values.ts_y_origin = -y;
+		gc_values.fill = GDK_TILED;
+		value_mask = GDK_GC_FILL | GDK_GC_TILE |
+			GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN;
+	} else {
+		gc_values.foreground = background->details->background_color;
+		gdk_rgb_find_color (gtk_widget_get_colormap (GTK_WIDGET (canvas)), &gc_values.foreground);
+		gc_values.fill = GDK_SOLID;
+		value_mask = GDK_GC_FILL | GDK_GC_FOREGROUND;
+	}
+		
+    	gc = gdk_gc_new_with_values (drawable, &gc_values, value_mask);
+	gdk_draw_rectangle (drawable, gc, TRUE, 0, 0, width, height);
+	g_object_unref (gc);
 
 	/* We don't want the draw from the base class to happen. */
 	g_signal_stop_emission_by_name (canvas, "draw_background");
@@ -1062,6 +1241,7 @@ eel_background_set_up_canvas (GtkWidget *widget)
 static void
 eel_widget_background_changed (GtkWidget *widget, EelBackground *background)
 {
+	eel_background_unrealize (background);
 	eel_background_set_widget_style (background, widget);
 	eel_background_set_up_canvas (widget);
 
@@ -1098,13 +1278,11 @@ eel_get_widget_background (GtkWidget *widget)
 	g_object_ref (background);
 	gtk_object_sink (GTK_OBJECT (background));
 	g_object_set_data_full (G_OBJECT (widget), "eel_background",
-				background, (GtkDestroyNotify) g_object_unref);
+				background, g_object_unref);
 
 	/* Arrange to get the signal whenever the background changes. */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (background),
-					       "appearance_changed",
-					       G_CALLBACK (eel_widget_background_changed),
-					       GTK_OBJECT (widget));
+	g_signal_connect_object (background, "appearance_changed",
+				 G_CALLBACK (eel_widget_background_changed), widget, G_CONNECT_SWAPPED);
 	eel_widget_background_changed (widget, background);
 
 	return background;
@@ -1134,25 +1312,6 @@ eel_gtk_widget_find_background_ancestor (GtkWidget *widget)
 	return NULL;
 }
 
-gboolean
-eel_background_is_too_complex_for_gtk_style (EelBackground *background)
-{
-	if (background == NULL) {
-		return FALSE;
-	}
-	
-	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
-	
-	if (background->details->image_uri != NULL) {
-		return TRUE;
-	}
-	if (!background->details->is_solid_color) {
-		return TRUE;
-	}
-	
-	return FALSE;
-}
-
 /* determine if a background is darker or lighter than average, to help clients know what
    colors to draw on top with */
 gboolean
