@@ -117,20 +117,14 @@ guint32             eel_gdk_color_to_rgb                   (const GdkColor
 GdkColor            eel_gdk_rgb_to_color                   (guint32              color);
 char *              eel_gdk_rgb_to_color_spec              (guint32              color);
 
+gboolean            eel_gdk_color_parse                    (const gchar         *spec,
+							    GdkColor            *color);
+
 /* Fill routines that take GdkRectangle parameters instead of four integers. */
 void                eel_fill_rectangle                     (GdkDrawable         *drawable,
 							    GdkGC               *gc,
 							    const GdkRectangle  *rectangle);
-void                eel_fill_rectangle_with_color          (GdkDrawable         *drawable,
-							    GdkGC               *gc,
-							    const GdkRectangle  *rectangle,
-							    guint32              rgb);
 gboolean            eel_gdk_color_is_dark                  (GdkColor            *color);
-void                eel_gdk_choose_foreground_color        (GdkColor            *preferred,
-							    GdkColor            *background);
-void                eel_gdk_gc_choose_foreground_color     (GdkGC               *gc,
-							    GdkColor            *preferred,
-							    GdkColor            *background);
 
 /* A routine to get a 50% gray stippled bitmap for use in some types of highlighting. */
 GdkBitmap *         eel_stipple_bitmap                     (void);
@@ -140,6 +134,8 @@ GdkBitmap *         eel_stipple_bitmap                     (void);
 gboolean            eel_rectangle_contains                 (const GdkRectangle  *rectangle,
 							    int                  x,
 							    int                  y);
+gboolean            eel_gdk_rectangle_contains_rectangle   (GdkRectangle         outer,
+							    GdkRectangle         inner);
 void                eel_rectangle_inset                    (GdkRectangle        *rectangle,
 							    int                  x,
 							    int                  y);
