@@ -55,9 +55,9 @@
 
 /* structure for the icon selection dialog */
 struct IconSelectionData {
+	GtkWidget *dialog;
         GtkWidget *icon_selection;
 	GtkWidget *file_entry;
-	GtkWidget *top_level_window;
 	GtkWindow *owning_window;
 	gboolean dismissed;
 	EelIconSelectionFunction selection_function;
@@ -645,25 +645,26 @@ eel_gnome_open_terminal (const char *command)
 }
 
 /* create a new icon selection dialog */
-
 static gboolean
 widget_destroy_callback (gpointer callback_data)
 {
-	IconSelectionData *selection_data = (IconSelectionData*) callback_data;
-	gtk_widget_destroy (selection_data->top_level_window);
+	IconSelectionData *selection_data;
+
+	selection_data = (IconSelectionData *) callback_data;
+	gtk_widget_destroy (selection_data->dialog);
 	g_free (selection_data);	
 	return FALSE;
 }
 
 /* set the image of the file object to the selected file */
 static void
-icon_selected_callback (GtkWidget *dialog, IconSelectionData *selection_data)
+icon_selected (IconSelectionData *selection_data)
 {
-	const gchar *icon_path;
+	const char *icon_path;
 	struct stat buf;
 	GtkWidget *entry;
 	
-	gnome_icon_selection_stop_loading ( GNOME_ICON_SELECTION (selection_data->icon_selection));
+	gnome_icon_selection_stop_loading (GNOME_ICON_SELECTION (selection_data->icon_selection));
 
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (selection_data->file_entry));
 	icon_path = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -681,59 +682,55 @@ icon_selected_callback (GtkWidget *dialog, IconSelectionData *selection_data)
 		selection_data->selection_function (icon_path, selection_data->callback_data);
 	}
 		
-	/* we have to get rid of the icon selection dialog, but we can't do it now, since the
-	 * file entry might still need it.  Do it when the next idle rolls around
+	/* We have to get rid of the icon selection dialog, but we can't do it now, since the
+	 * file entry might still need it. Do it when the next idle rolls around
 	 */ 
 	selection_data->dismissed = TRUE;
-	selection_data->top_level_window = dialog;
-	gtk_idle_add ((GtkFunction) widget_destroy_callback, selection_data);
+	gtk_idle_add (widget_destroy_callback, selection_data);
 }
 
 /* handle the cancel button being pressed */
 static void
-icon_cancel_pressed (GtkWidget *dialog, IconSelectionData *selection_data)
+icon_cancel_pressed (IconSelectionData *selection_data)
 {
 	/* nothing to do if it's already been dismissed */
 	if (selection_data->dismissed) {
 		return;
 	}
 	
-	gnome_icon_selection_stop_loading(GNOME_ICON_SELECTION (selection_data->icon_selection));	
-	/* remove pending idle routine, if necessary */
+	gtk_widget_destroy (selection_data->dialog);
 	g_free (selection_data);
-	gtk_widget_destroy (dialog);
 }
 
 /* handle an icon being selected by updating the file entry */
 static void
-list_icon_selected_callback (GnomeIconList *gil, gint num, GdkEvent *event, IconSelectionData *selection_data)
+list_icon_selected_callback (GnomeIconList *gil, int num, GdkEvent *event, IconSelectionData *selection_data)
 {
-	const gchar *icon;
+	const char *icon;
 	GtkWidget *entry;
 	
 	icon = gnome_icon_selection_get_icon (GNOME_ICON_SELECTION (selection_data->icon_selection), TRUE);
-
 	if (icon != NULL) {
 		entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (selection_data->file_entry));
-		gtk_entry_set_text(GTK_ENTRY (entry), icon);
+		gtk_entry_set_text (GTK_ENTRY (entry), icon);
 	}
 
 	/* handle double-clicks as if the user pressed OK */
-	if(event && event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *)event)->button == 1) {
-		icon_selected_callback (selection_data->file_entry, selection_data);
+	if (event != NULL && event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *) event)->button == 1) {
+		icon_selected (selection_data);
 	}
 }
 
 static void
-dialog_response_callback (GtkWidget *dialog, gint response_id, IconSelectionData *selection_data)
+dialog_response_callback (GtkWidget *dialog, int response_id, IconSelectionData *selection_data)
 {
 	switch (response_id) {
 	case GTK_RESPONSE_OK:
-		icon_selected_callback (dialog, selection_data);
+		icon_selected (selection_data);
 		break;
 	case GTK_RESPONSE_CANCEL:
 	case GTK_RESPONSE_DELETE_EVENT:
-		icon_cancel_pressed (dialog, selection_data);
+		icon_cancel_pressed (selection_data);
 		break;
 	default:
 		break;
@@ -742,26 +739,25 @@ dialog_response_callback (GtkWidget *dialog, gint response_id, IconSelectionData
 
 /* handle the file entry changing */
 static void
-entry_activated (GtkWidget *widget, IconSelectionData *selection_data)
+entry_activated_callback (GtkWidget *widget, IconSelectionData *selection_data)
 {
 	struct stat buf;
-	GtkWidget *icon_selection;
-	const gchar *filename;
+	GnomeIconSelection *icon_selection;
+	const char *filename;
 
 	filename = gtk_entry_get_text (GTK_ENTRY (widget));
-	if (!filename) {
+	if (filename == NULL) {
 		return;
 	}
 	
-	stat (filename, &buf);
-	if (S_ISDIR (buf.st_mode)) {
-		icon_selection = selection_data->icon_selection;
-		gnome_icon_selection_clear (GNOME_ICON_SELECTION (icon_selection), TRUE);
-		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (icon_selection), filename);
-		gnome_icon_selection_show_icons(GNOME_ICON_SELECTION (icon_selection));
+	if (stat (filename, &buf) == 0 && S_ISDIR (buf.st_mode)) {
+		icon_selection = GNOME_ICON_SELECTION (selection_data->icon_selection);
+		gnome_icon_selection_clear (icon_selection, TRUE);
+		gnome_icon_selection_add_directory (icon_selection, filename);
+		gnome_icon_selection_show_icons(icon_selection);
 	} else {
 		/* We pretend like ok has been called */
-		icon_selected_callback (selection_data->file_entry, selection_data);
+		icon_selected (selection_data);
 	}
 }
 
@@ -785,21 +781,22 @@ eel_gnome_icon_selector_new (const char *title,
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
-	icon_selection = gnome_icon_selection_new();
+	icon_selection = gnome_icon_selection_new ();
 
 	file_entry = gnome_file_entry_new (NULL,NULL);
 	
 	selection_data = g_new0 (IconSelectionData, 1);
+	selection_data->dialog = dialog;
  	selection_data->icon_selection = icon_selection;
  	selection_data->file_entry = file_entry;
  	selection_data->owning_window = owner;
 	selection_data->selection_function = selected;
  	selection_data->callback_data = callback_data;
  	
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    file_entry, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    icon_selection, TRUE, TRUE, 0);
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
@@ -829,7 +826,7 @@ eel_gnome_icon_selector_new (const char *title,
 				G_CALLBACK (list_icon_selected_callback), selection_data);
 
 	g_signal_connect (entry, "activate",
-			  G_CALLBACK (entry_activated), selection_data);
+			  G_CALLBACK (entry_activated_callback), selection_data);
 
 	return dialog;
 }
