@@ -33,6 +33,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtktreeview.h>
 #include <gtk/gtkwindow.h>
 #include <eel/eel-gdk-extensions.h>
 
@@ -67,14 +68,6 @@ void                  eel_gtk_signal_connect_while_realized           (GtkObject
 								       GtkSignalFunc         callback,
 								       gpointer              callback_data,
 								       GtkWidget            *realized_widget);
-void                  eel_nullify_when_destroyed                      (gpointer              object_pointer_address);
-void                  eel_nullify_cancel                              (gpointer              object_pointer_address);
-
-/* list of GtkObject */
-GList *               eel_gtk_object_list_ref                         (GList                *list);
-void                  eel_gtk_object_list_unref                       (GList                *list);
-void                  eel_gtk_object_list_free                        (GList                *list);
-GList *               eel_gtk_object_list_copy                        (GList                *list);
 
 /* GtkWidget */
 void                  eel_gtk_widget_set_shown                        (GtkWidget            *widget,
@@ -151,4 +144,9 @@ void                  eel_gtk_adjustment_clamp_value                  (GtkAdjust
 /* Client-side drawing with Pango */
 PangoContext *        eel_gtk_widget_get_pango_ft2_context            (GtkWidget            *widget);
 
+/* GtkTreeView */
+gboolean              eel_gtk_tree_view_cell_is_completely_visible    (GtkTreeView          *tree_view,
+								       GtkTreePath          *path,
+								       GtkTreeViewColumn    *column);
+
 #endif /* EEL_GTK_EXTENSIONS_H */
