@@ -23,14 +23,13 @@
 */
 
 #include <config.h>
-
 #include "eel-wrap-table.h"
 
-#include "eel-gtk-macros.h"
-#include "eel-gtk-extensions.h"
 #include "eel-art-extensions.h"
 #include "eel-art-gtk-extensions.h"
-
+#include "eel-gtk-extensions.h"
+#include "eel-gtk-macros.h"
+#include "eel-types.h"
 #include <gtk/gtkmain.h>
 #include <gtk/gtkviewport.h>
 #include <libgnome/gnome-i18n.h>
@@ -131,47 +130,60 @@ eel_wrap_table_class_init (EelWrapTableClass *wrap_table_class)
 	container_class->forall = eel_wrap_table_forall;
 	container_class->child_type = eel_wrap_table_child_type;
 
+	/* Register some the enum types we need */
+	eel_type_init ();
+
 	/* Arguments */
-	g_object_class_install_property (
-		gobject_class,
-		PROP_X_SPACING,
-		g_param_spec_uint ("x_spacing",
-				   _("X spacing"),
-				   _("The amount of horizontal spacing."),
-				   0,
-				   G_MAXINT,
-				   0,
-				   G_PARAM_READWRITE));
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_X_SPACING,
+		 g_param_spec_uint ("x_spacing",
+				    _("horizontal spacing"),
+				    _("The amount of horizontal spacing."),
+				    0,
+				    G_MAXINT,
+				    0,
+				    G_PARAM_READWRITE));
 
-	g_object_class_install_property (
-		gobject_class,
-		PROP_Y_SPACING,
-		g_param_spec_uint ("y_spacing",
-				   _("Y spacing"),
-				   _("The amount of vertical spacing."),
-				   0,
-				   G_MAXINT,
-				   0,
-				   G_PARAM_READWRITE));
-
-#if GNOME2_CONVERSION_COMPLETE
-	gtk_object_add_arg_type ("EelWrapTable::x_justification",
-				 G_TYPE_ENUM,
-				 GTK_PROP_READWRITE,
-				 PROP_X_JUSTIFICATION);
-	gtk_object_add_arg_type ("EelWrapTable::y_justification",
-				 G_TYPE_ENUM,
-				 GTK_PROP_READWRITE,
-				 PROP_Y_JUSTIFICATION);
-#endif
-	g_object_class_install_property (
-		gobject_class,
-		PROP_HOMOGENEOUS,
-		g_param_spec_boolean ("homogenous",
-				      _("homogenous"),
-				      _("homogeneous layout"),
-				      FALSE,
-				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_Y_SPACING,
+		 g_param_spec_uint ("y_spacing",
+				    _("vertical spacing"),
+				    _("The amount of vertical spacing."),
+				    0,
+				    G_MAXINT,
+				    0,
+				    G_PARAM_READWRITE));
+	
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_X_JUSTIFICATION,
+		 g_param_spec_enum ("x_justification",
+				    _("horizontal justification"),
+				    _("Left, right, or center justification"),
+				    EEL_TYPE_JUSTIFICATION,
+				    EEL_JUSTIFICATION_BEGINNING,
+				    G_PARAM_READWRITE));
+				    
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_Y_JUSTIFICATION,
+		 g_param_spec_enum ("y_justification",
+				    _("vertical justification"),
+				    _("Top, bottom, or middle justification"),
+				    EEL_TYPE_JUSTIFICATION,
+				    EEL_JUSTIFICATION_BEGINNING,
+				    G_PARAM_READWRITE));
+				    
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_HOMOGENEOUS,
+		 g_param_spec_boolean ("homogenous",
+				       _("homogenous"),
+				       _("homogeneous layout"),
+				       FALSE,
+				       G_PARAM_READWRITE));
 }
 
 void
